package com.neurosdk2;

import android.util.Log;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.neurosdk2.neuro.ISensor;
import com.neurosdk2.neuro.Scanner;
import com.neurosdk2.neuro.Sensor;
import com.neurosdk2.neuro.types.ParameterInfo;
import com.neurosdk2.neuro.types.SensorCommand;
import com.neurosdk2.neuro.types.SensorFamily;
import com.neurosdk2.neuro.types.SensorFeature;

import static com.neurosdk2.Utils.ConvertReactToIntArray;
import static com.neurosdk2.Utils.ConvertStrToByteArray;
import static com.neurosdk2.Utils.ConvertResistChannelsData;
import static com.neurosdk2.Utils.ConvertSignalChannelsData;
import com.neurosdk2.neuro.types.ResistChannelsData;
import com.neurosdk2.neuro.types.SignalChannelsData;
import com.neurosdk2.neuro.NeuroEEGSignalProcessing;
import com.neurosdk2.neuro.types.NeuroEEGAmplifierParam;
import com.neurosdk2.neuro.types.ProcessSignalResult;


import java.util.HashMap;
import java.util.Objects;

public class SensorHolder {

  private final HashMap<String, AFullSensor> _sensors = new HashMap<>();
  protected ReactApplicationContext _reactContext;

  public SensorHolder(ReactApplicationContext context)
  {
    _reactContext = context;
  }
  public synchronized void addSensor(String guid, Sensor sensor){
    SensorFamily sensFamily = sensor.getSensFamily();
    switch (sensFamily){
      case SensorUnknown:
        break;

      case SensorLECallibri:
      case SensorLEKolibri:
        _sensors.put(guid, new CallibriImpl(sensor, guid, _reactContext));
        break;


      case SensorLEBrainBit:
        _sensors.put(guid, new BrainBitImpl(sensor, guid, _reactContext));
        break;


      case SensorLEBrainBitBlack:
        _sensors.put(guid, new BrainBitBlackImpl(sensor, guid, _reactContext));
        break;


      case SensorLENeuroEEG:
        _sensors.put(guid, new NeuroEEGImpl(sensor, guid, _reactContext));
        break;


      case SensorLEBrainBit2:
      case SensorLEBrainBitFlex:
      case SensorLEBrainBitPro:
        _sensors.put(guid, new BrainBit2Impl(sensor, guid, _reactContext));
        break;


      case SensorLEHeadPhones2:
        _sensors.put(guid, new Headphones2Impl(sensor, guid, _reactContext));
        break;

    }
  }

  // <editor-fold> Sensor
  public synchronized void closeSensor(String guid) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.closeSensor();
      _sensors.remove(guid);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public synchronized void connectSensor(String guid, Promise promise) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.connectSensor();
      promise.resolve("success");
    }
    catch (Exception ex){
      promise.reject("connectSensor", ex.getLocalizedMessage());
    }
  }

  public void disconnectSensor(String guid, Promise promise) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.disconnectSensor();
      promise.resolve("success");
    }
    catch (Exception ex){
      promise.reject("disconnectSensor", ex.getLocalizedMessage());
    }
  }

  public WritableArray supportedFeatures(String guid) {
    WritableArray features = Arguments.createArray();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.supportedFeatures(features);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return features;
  }

  public boolean isSupportedFeature(String guid, double feature) {
    boolean isSupported = false;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      isSupported = sensor.isSupportedFeature((int)feature);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return isSupported;
  }

  public WritableArray supportedCommands(String guid) {
    WritableArray commands = Arguments.createArray();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.supportedCommands(commands);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return commands;
  }

  public boolean isSupportedCommand(String guid, double command) {
    boolean isSupported = false;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      isSupported = sensor.isSupportedCommand((int)command);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return isSupported;
  }

  public WritableArray supportedParameters(String guid) {
    WritableArray params = Arguments.createArray();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.supportedParameters(params);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return params;
  }

  public boolean isSupportedParameter(String guid, double parameter) {
    boolean isSupported = false;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      isSupported = sensor.isSupportedParameter((int)parameter);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return isSupported;
  }

  public synchronized void execCommand(String guid, double command, Promise promise) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.execCommand((int)command);
      promise.resolve("success");
    }
    catch (Exception ex){
      ex.printStackTrace();
      promise.reject("execCommand", ex.getLocalizedMessage());
    }
  }

  public String getName(String guid) {
    String result = "";
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getName();
      Log.d("getName", result);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public void setName(String guid, String name) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setName(name);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public double getState(String guid) {
    double result = 1;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getState();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public String getAddress(String guid) {
    String result = "";
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getAddress();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public String getSerialNumber(String guid) {
    String result = "";
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getSerialNumber();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public void setSerialNumber(String guid, String sn) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setSerialNumber(sn);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public double getBattPower(String guid) {
    double result = 0;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getBattPower();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public double getSamplingFrequency(String guid) {
    double result = 0xFF;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getSamplingFrequency();
      Log.d("getSamplingFrequency", result + "");
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public double getGain(String guid) {
    double result = 11;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getGain();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public double getDataOffset(String guid) {
    double result = 0xFF;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getDataOffset();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public double getFirmwareMode(String guid) {
    double result = 0;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getFirmwareMode();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public WritableMap getVersion(String guid) {
    WritableMap res = Arguments.createMap();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.getVersion(res);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return res;
  }

  public double getChannelsCount(String guid) {
    double result = 0;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getChannelsCount();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public double getSensFamily(String guid) {
    double result = 0;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getSensFamily();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public void set66HzFEnabled(String guid, boolean enabled){
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.set66HzFEnabled(enabled);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public boolean get66HzFEnabled(String guid){
    boolean result = true;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.get66HzFEnabled();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }
  // </editor-fold>

  public double getAmpMode(String guid) {
    double result = 0;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getAmpMode();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public double getSamplingFrequencyResist(String guid) {
    double result = 0xFF;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getSamplingFrequencyResist();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }


  // <editor-fold> BrainBitBlack
  
  public double getSamplingFrequencyFPG(String guid) {
    double result = 0xFF;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getSamplingFrequencyFPG();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public double getIrAmplitude(String guid) {
    double result = 0xFF;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getIrAmplitude();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public void setIrAmplitude(String guid, double amp) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setIrAmplitude((int)amp);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public double getRedAmplitude(String guid) {
    double result = 0xFF;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getRedAmplitude();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public void setRedAmplitude(String guid, double amp) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setRedAmplitude((int)amp);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public void pingNeuroSmart(String guid, double marker, Promise promise) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.pingNeuroSmart(marker);
      promise.resolve("success");
    }
    catch (Exception ex){
      ex.printStackTrace();
      promise.reject("pingNeuroSmart", ex.getLocalizedMessage());
    }
  }
  // </editor-fold>


  // <editor-fold> Callibri

  public boolean isSupportedFilter(String guid, double filter) {
    boolean result = false;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.isSupportedFilter((int)filter);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public WritableArray getSupportedFilters(String guid) {
    WritableArray filters = Arguments.createArray();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.getSupportedFilters(filters);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return filters;
  }

  public WritableArray getHardwareFilters(String guid) {
    WritableArray filters = Arguments.createArray();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.getHardwareFilters(filters);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return filters;
  }

  public void setHardwareFilters(String guid, ReadableArray filters) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setHardwareFilters(filters);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public void setFirmwareMode(String guid, double mode) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setFirmwareMode((int)mode);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public void setSamplingFrequency(String guid, double sf) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setSamplingFrequency((int)sf);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public void setGain(String guid, double gain) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setGain((int)gain);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public void setDataOffset(String guid, double offset) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setDataOffset((int)offset);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public double getExtSwInput(String guid) {
    double result = 0xFF;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getExtSwInput();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public void setExtSwInput(String guid, double extSwInp) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setExtSwInput((int)extSwInp);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public double getADCInput(String guid) {
    double result = 0;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getADCInput();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public void setADCInput(String guid, double adcInp) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setADCInput((int)adcInp);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public WritableMap getStimulatorMAState(String guid) {
    WritableMap map = Arguments.createMap();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.getStimulatorMAState(map);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return map;
  }

  public WritableMap getStimulatorParam(String guid) {
    WritableMap map = Arguments.createMap();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.getStimulatorParam(map);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return map;
  }

  public void setStimulatorParam(String guid, ReadableMap param) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setStimulatorParam(param);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public WritableMap getMotionAssistantParam(String guid) {
    WritableMap map = Arguments.createMap();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.getMotionAssistantParam(map);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return map;
  }

  public void setMotionAssistantParam(String guid, ReadableMap param) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setMotionAssistantParam(param);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public WritableMap getMotionCounterParam(String guid) {
    WritableMap map = Arguments.createMap();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.getMotionCounterParam(map);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return map;
  }

  public void setMotionCounterParam(String guid, ReadableMap param) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setMotionCounterParam(param);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public double getMotionCounter(String guid) {
    double result = 0;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getMotionCounter();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public double getColor(String guid) {
    double result = 0;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getColor();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public boolean getMEMSCalibrateState(String guid) {
    boolean result = false;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getMEMSCalibrateState();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public double getSamplingFrequencyResp(String guid) {
    double result = 0xFF;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getSamplingFrequencyResp();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public double getSamplingFrequencyEnvelope(String guid) {
    double result = 0xFF;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getSamplingFrequencyEnvelope();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public double getSignalType(String guid) {
    double result = 0;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getSignalType();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public void setSignalType(String guid, double type) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setSignalType((int)type);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public double getElectrodeState(String guid) {
    double result = 0;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getElectrodeState();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  // </editor-fold>


public double getAccSens(String guid) {
  double result = 0;
  try {
    AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
    result = sensor.getAccSens();
  }
  catch (Exception ex){
    ex.printStackTrace();
  }
  return result;
}

public void setAccSens(String guid, double accSens) {
  try {
    AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
    sensor.setAccSens((int)accSens);
  }
  catch (Exception ex){
    ex.printStackTrace();
  }
}

public double getGyroSens(String guid) {
  double result = 0;
  try {
    AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
    result = sensor.getGyroSens();
  }
  catch (Exception ex){
    ex.printStackTrace();
  }
  return result;
}

public void setGyroSens(String guid, double gyroSens) {
  try {
    AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
    sensor.setGyroSens((int)gyroSens);
  }
  catch (Exception ex){
    ex.printStackTrace();
  }
}

public double getSamplingFrequencyMEMS(String guid) {
  double result = 0xFF;
  try {
    AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
    result = sensor.getSamplingFrequencyMEMS();
  }
  catch (Exception ex){
    ex.printStackTrace();
  }
  return result;
}


  // <editor-fold> NeuroEEG

  public double getSurveyId(String guid) {
    double result = 0;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.getSurveyId();
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public void setSurveyId(String guid, double id) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setSurveyId((int)id);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public WritableArray getSupportedChannels(String guid) {
    WritableArray channels = Arguments.createArray();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.getSupportedChannels(channels);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return channels;
  }

  public WritableMap getAmplifierParam(String guid) {
    WritableMap map = Arguments.createMap();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.getAmplifierParam(map);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return map;
  }

  public void setAmplifierParam(String guid, ReadableMap param) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setAmplifierParam(param);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }


  public WritableMap getFSStatus(String guid) {
    WritableMap map = Arguments.createMap();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.getFSStatus(map);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return map;
  }

  public WritableMap getFSDiskInfo(String guid) {
    WritableMap map = Arguments.createMap();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.getFSDiskInfo(map);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return map;
  }

  public WritableMap readFileInfo(String guid, String fileName) {
    WritableMap infoMap = Arguments.createMap();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.readFileInfo(fileName, infoMap);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return infoMap;
  }

  public WritableArray readFileInfoAll(String guid, int maxFiles) {
    WritableArray infoArray = Arguments.createArray();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.readFileInfoAll(maxFiles, infoArray);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return infoArray;
  }

  public void writeFile(String guid, String fileName, ReadableArray data, int offset, Promise promise) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.writeFile(fileName, data, offset);
      promise.resolve("success");
    }
    catch (Exception ex){
      ex.printStackTrace();
      promise.reject(ex);
    }
  }

  public void readFile(String guid, String fileName, int sizeData, int offset, Promise promise) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      promise.resolve(sensor.readFile(fileName, sizeData, offset));
    }
    catch (Exception ex){
      ex.printStackTrace();
      promise.reject(ex);
    }
  }

  public void deleteFile(String guid, String fileName, Promise promise) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.deleteFile(fileName);
      promise.resolve("success");
    }
    catch (Exception ex){
      ex.printStackTrace();
      promise.reject(ex);
    }
  }

  public void deleteAllFiles(String guid, String fileExt, Promise promise) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.deleteAllFiles(fileExt);
      promise.resolve("success");
    }
    catch (Exception ex){
      ex.printStackTrace();
      promise.reject(ex);
    }
  }
  
  public int readFileCRC32(String guid, String fileName, int totalSize, int offset) {
    int result = 0;
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      result = sensor.readFileCRC32(fileName, totalSize, offset);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return result;
  }

  public void fileStreamAutosave(String guid, String fileName) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.fileStreamAutosave(fileName);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public void fileStreamRead(String guid, String fileName, int totalSize, int offset) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.fileStreamRead(fileName, totalSize, offset);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }


  // </editor-fold>



  public WritableMap getAmplifierParamBB2(String guid) {
    WritableMap map = Arguments.createMap();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.getAmplifierParamBB2(map);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return map;
  }

  public double setAmplifierParamBB2(String guid, ReadableMap param) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setAmplifierParamBB2(param);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return 0;
  }


  public WritableMap getAmplifierParamHP2(String guid) {
    WritableMap map = Arguments.createMap();
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.getAmplifierParamHP2(map);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
    return map;
  }

  public void setAmplifierParamHP2(String guid, ReadableMap param) {
    try {
      AFullSensor sensor = Objects.requireNonNull(_sensors.get(guid));
      sensor.setAmplifierParamHP2(param);
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }


  private NeuroEEGSignalProcessing process = null;

  public void startProcessingData(ReadableMap param){
    try{
          process = new NeuroEEGSignalProcessing(new NeuroEEGAmplifierParam(
            param.getBoolean("ReferentResistMesureAllow"),
            (int) param.getInt("Frequency"),
            (int) param.getInt("ReferentMode"),
            ConvertReactToIntArray(param.getArray("ChannelMode")),
            ConvertReactToIntArray(param.getArray("ChannelGain"))
          ));
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

  public void parseRawSignal(ReadableArray data, Promise promise){
    try{
      byte[] byteData = new byte[data.size()];
      for (int i = 0; i < data.size(); i++)
      {
        byteData[i] = (byte)data.getInt(i);
      }
      WritableMap map = Arguments.createMap(); 
      ProcessSignalResult result = process.parseRawSignal(byteData);  
      map.putInt("SzReady", result.getSzReady());

      WritableArray signalArray = Arguments.createArray();
      for (SignalChannelsData signal : result.getSignal()) {
        WritableMap smap = Arguments.createMap();
        ConvertSignalChannelsData(signal, smap);
        signalArray.pushMap(smap);
      }
      WritableArray resistArray = Arguments.createArray();
      for (ResistChannelsData resist : result.getResist()) {
        WritableMap rmap = Arguments.createMap();
        ConvertResistChannelsData(resist, rmap);
        resistArray.pushMap(rmap);
      }

      map.putArray("Signal", signalArray);
      map.putArray("Resist", resistArray);
      promise.resolve(map);
    }
    catch (Exception ex){
      promise.reject(ex);
    }
  }

  public void stopProcessingData(){
    try{
      process.close();
      process = null;
    }
    catch (Exception ex){
      ex.printStackTrace();
    }
  }

}
