import Sensor from "../Sensor";
import type { EmitterSubscription } from "react-native";
import { Neurosdk2 } from "../ModuleResolver";
import type { SensorAmpMode, SensorSamplingFrequency, EEGChannelInfo, NeuroEEGAmplifierParam, SignalChannelsData, ResistChannelsData,

  SensorFileData, SensorFileInfo, NeuroEEGFSStatus, SensorDiskInfo, ProcessSignalResult

} from "../NativeNeurosdk2";

import { Buffer } from 'buffer'


function _convertStrToUint8Arr(str: String): Uint8Array {
  return Uint8Array.from(Buffer.from(str, 'binary'));
}

export default class NeuroEEGSensor extends Sensor {

    private ampModeChanged: EmitterSubscription | undefined;
    private signalDataReceived: EmitterSubscription | undefined;
    private resistDataReceived: EmitterSubscription | undefined;
    private signalResistReceived: EmitterSubscription | undefined;

    private signalRawDataReceived: EmitterSubscription | undefined;
    private fileStreamDataReceived: EmitterSubscription | undefined;


    constructor(guid: string) {
      super(guid)
    }

    static getMaxChCount(): number{
      return 24;
    }

    getSurveyId(): number{
      return Neurosdk2.getSurveyId(this.guid);
    }
    setSurveyId(id: number){
      Neurosdk2.setSurveyId(this.guid, id);
    }

    getAmpMode(): SensorAmpMode{
      return Neurosdk2.getAmpMode(this.guid);
    }

    getSupportedChannels(): Array<EEGChannelInfo>{
      return Neurosdk2.getSupportedChannels(this.guid);
    }

    getAmplifierParam(): NeuroEEGAmplifierParam{
      return Neurosdk2.getAmplifierParam(this.guid);
    }
    setAmplifierParam(param: NeuroEEGAmplifierParam){
      Neurosdk2.setAmplifierParam(this.guid, param);
    }

    getSamplingFrequencyResist(): SensorSamplingFrequency{
      return Neurosdk2.getSamplingFrequencyResist(this.guid);
    }

    getFSStatus(): NeuroEEGFSStatus{
      return Neurosdk2.getFSStatus(this.guid);
    }

    getFSDiskInfo(): SensorDiskInfo{
      return Neurosdk2.getFSDiskInfo(this.guid);
    }

    readFileInfo(fileName: string) : SensorFileInfo{
      return Neurosdk2.readFileInfo(this.guid, fileName);
    }

    readFileInfoAll(maxFiles: number):  Array<SensorFileInfo> {
      return Neurosdk2.readFileInfoAll(this.guid, maxFiles);
    }

    writeFile(fileName: string, data: Uint8Array, offset: number = 0): Promise<string>{
      return Neurosdk2.writeFile(this.guid, fileName, Array.from(data), offset);
    }

    readFile(fileName: string, sizeData: number = parseInt("0xFFFFFFFF", 16), offset: number = 0): Promise<Uint8Array> {
      return new Promise<Uint8Array>(async (resolve, reject) => {
        try {
          let result = await Neurosdk2.readFile(this.guid, fileName, sizeData, offset);
          resolve(_convertStrToUint8Arr(result));
        } catch (error) {
          reject(error)
        }
      });
    }

    deleteFile(fileName: string): Promise<string>{
      return Neurosdk2.deleteFile(this.guid, fileName);
    }

    deleteAllFiles(fileExt: string): Promise<string>{
      return Neurosdk2.deleteAllFiles(this.guid, fileExt);
    }

    readFileCRC32(fileName: string, totalSize: number, offset: number): number{
      return Neurosdk2.readFileCRC32(this.guid, fileName, totalSize, offset);
    }

    fileStreamAutosave(fileName: string){
      return Neurosdk2.fileStreamAutosave(this.guid, fileName);
    }

    fileStreamRead(fileName: string, totalSize: number = 0, offset: number = 0){
      Neurosdk2.fileStreamRead(this.guid, fileName, totalSize, offset);
    }

    AddAmpModeChanged(callback:(ampMode:SensorAmpMode)=>void){
      this.ampModeChanged = this.nativeEventEmitter.addListener(
          Neurosdk2.AMPMODE_CHANGED_EVENT_NAME,
        (ampMode) => { if(ampMode.GUID === this.guid) callback(ampMode.data)}
      );
    }
    RemoveAmpModeChanged(){
      if(this.ampModeChanged != undefined)
        this.ampModeChanged.remove();
    }

    AddSignalDataReceived(callback:(signalData:Array<SignalChannelsData>)=>void){
      this.signalDataReceived = this.nativeEventEmitter.addListener(
          Neurosdk2.NEURRO_EEG_SIGNAL_DATA_EVENT_NAME,
        (signalData) => { if(signalData.GUID === this.guid) callback(signalData.data)}
      );
    }
    RemoveSignalDataReceived(){
      if(this.signalDataReceived != undefined)
        this.signalDataReceived.remove();
    }

    AddResistDataReceived(callback:(resistData:Array<ResistChannelsData>)=>void){
      this.resistDataReceived = this.nativeEventEmitter.addListener(
          Neurosdk2.NEURRO_EEG_RESIST_DATA_EVENT_NAME,
        (resistData) => { if(resistData.GUID === this.guid) callback(resistData.data)}
      );
    }
    RemoveResistDataReceived(){
      if(this.resistDataReceived != undefined)
        this.resistDataReceived.remove();
    }

    AddSignalResistReceived(callback:(signalData:Array<SignalChannelsData>, resistData:Array<ResistChannelsData>)=>void){
      this.signalResistReceived = this.nativeEventEmitter.addListener(
          Neurosdk2.NEURRO_EEG_SIGNAL_RESIST_DATA_EVENT_NAME,
        (data) => { if(data.GUID === this.guid) callback(data.data.signalData, data.data.resistData)}
      );
    }
    RemoveSignalResistReceived(){
      if(this.signalResistReceived != undefined)
        this.signalResistReceived.remove();
    }

    AddSignalRawDataReceived(callback:(resistData:Uint8Array)=>void){
      this.signalRawDataReceived = this.nativeEventEmitter.addListener(
          Neurosdk2.NEURRO_EEG_RAW_DATA_EVENT_NAME,
        (resistData) => { if(resistData.GUID === this.guid) callback(resistData.data)}
      );
    }
    RemoveSignalRawDataReceived(){
      if(this.signalRawDataReceived != undefined)
        this.signalRawDataReceived.remove();
    }

    AddFileStreamDataReceived(callback:(fileData:Array<SensorFileData>)=>void){
      this.fileStreamDataReceived = this.nativeEventEmitter.addListener(
          Neurosdk2.NEURRO_EEG_FILE_STREAM_DATA_EVENT_NAME,
        (fileData) => { if(fileData.GUID === this.guid) callback(fileData.data)}
      );
    }
    RemoveFileStreamDataReceived(){
      if(this.fileStreamDataReceived != undefined)
        this.fileStreamDataReceived.remove();
    }

    static StartProcessingData(param: NeuroEEGAmplifierParam){
      Neurosdk2.startProcessingData(param);
    }
    
    static ParseRawSignal(data: Uint8Array): Promise<ProcessSignalResult>{
      return Neurosdk2.parseRawSignal(Array.from(data));
    }
    
    static StopProcessingData(){
      Neurosdk2.stopProcessingData();
    }

}